@@ -22,7 +22,7 @@ from fastworkflow.model_pipeline_training import (
 )
 
 from fastworkflow.train.generate_synthetic import generate_diverse_utterances
-from fastworkflow.utils.fuzzy_match import find_best_match
+from fastworkflow.utils.fuzzy_match import find_best_matches
 from fastworkflow.utils.signatures import InputForParamExtraction
 
 
@@ -85,6 +85,16 @@ class CommandNamePrediction:
             for fully_qualified_command_name in valid_command_names
         }
 
+        if nlu_pipeline_stage == NLUPipelineStage.INTENT_AMBIGUITY_CLARIFICATION:
+            # what_can_i_do is special in INTENT_AMBIGUITY_CLARIFICATION
+            # We will not predict, just match plain utterances with exact or fuzzy match
+            command_name_dict |= {
+                plain_utterance: 'IntentDetection/what_can_i_do'
+                for plain_utterance in crd.command_directory.map_command_2_utterance_metadata[
+                    'IntentDetection/what_can_i_do'
+                ].plain_utterances
+            }
+
         if nlu_pipeline_stage != NLUPipelineStage.INTENT_DETECTION:
             # abort is special. 
             # We will not predict, just match plain utterances with exact or fuzzy match
@@ -105,22 +115,22 @@ class CommandNamePrediction:
                 ].plain_utterances
             }
 
-        command_name = None
         # See if the command starts with a command name followed by a space
         tentative_command_name = command.split(" ", 1)[0]
         normalized_command_name = tentative_command_name.lower()
+        command_name = None
         if normalized_command_name in command_name_dict:
             command_name = normalized_command_name
             command = command.replace(f"{tentative_command_name}", "").strip().replace("  ", " ")
-
-        # Use Levenshtein distance for fuzzy matching with the full command part after @
-        matched_command, distance = find_best_match(
-            command,
-            command_name_dict.keys(),
-            threshold=0.3  # Adjust threshold as needed
-        )
-        if matched_command:
-            command_name = matched_command
+        else:
+            # Use Levenshtein distance for fuzzy matching with the full command part after @
+            best_matched_commands, _ = find_best_matches(
+                command.replace(" ", "_"),
+                command_name_dict.keys(),
+                threshold=0.3  # Adjust threshold as needed
+            )
+            if best_matched_commands:
+                command_name = best_matched_commands[0]
 
         if nlu_pipeline_stage == NLUPipelineStage.INTENT_DETECTION:
             if not command_name:
@@ -132,7 +142,7 @@ class CommandNamePrediction:
                     if len(predictions)==1:
                         command_name = predictions[0].split('/')[-1]
                     else:
-                    # If confidence is low, treat as ambiguous command (type 1)
+                        # If confidence is low, treat as ambiguous command (type 1)
                         error_msg = self._formulate_ambiguous_command_error_message(predictions)
                         # Store suggested commands
                         self._store_suggested_commands(self.path, predictions, 1)
@@ -141,13 +151,8 @@ class CommandNamePrediction:
         elif nlu_pipeline_stage in (
             NLUPipelineStage.INTENT_AMBIGUITY_CLARIFICATION,
             NLUPipelineStage.INTENT_MISUNDERSTANDING_CLARIFICATION
-        ) and command_name != 'abort':
-            if command_name:
-                command = self.cme_workflow.context["command"]
-                store_utterance_cache(self.path, command, command_name, modelpipeline)
-            else:
-                error_msg = "Enter a valid command"
-                return CommandNamePrediction.Output(error_msg=error_msg)
+        ) and not command_name:
+            command_name = "what_can_i_do"
 
         if not command_name or command_name == "wildcard":
             fully_qualified_command_name=None
@@ -159,6 +164,19 @@ class CommandNamePrediction:
                 fully_qualified_command_name in crd.get_command_names('ErrorCorrection')
             )
 
+        if (
+            nlu_pipeline_stage
+            in (
+                NLUPipelineStage.INTENT_AMBIGUITY_CLARIFICATION,
+                NLUPipelineStage.INTENT_MISUNDERSTANDING_CLARIFICATION,
+            )
+            and not fully_qualified_command_name.endswith('abort')
+            and not fully_qualified_command_name.endswith('what_can_i_do')
+            and not fully_qualified_command_name.endswith('you_misunderstood')
+        ):
+            command = self.cme_workflow.context["command"]
+            store_utterance_cache(self.path, command, command_name, modelpipeline)
+
         return CommandNamePrediction.Output(
             command_name=fully_qualified_command_name,
             is_cme_command=is_cme_command
@@ -279,7 +297,7 @@ class CommandNamePrediction:
     def _formulate_ambiguous_command_error_message(route_choice_list: list[str]) -> str:
         command_list = (
             "\n".join([
-                f"{route_choice}"
+                f"{route_choice.split('/')[-1].lower()}"
                 for route_choice in route_choice_list if route_choice != 'wildcard'
             ])
         )
@@ -287,7 +305,7 @@ class CommandNamePrediction:
         return (
             "The command is ambiguous. Please select from these possible options:\n"
             f"{command_list}\n\n"
-            "or type 'none of these' to see all commands\n"
+            "or type 'what can i do' to see all commands\n"
             "or type 'abort' to cancel"
         )
 
@@ -325,13 +343,10 @@ class ParameterExtraction:
         else:
             stored_missing_fields = []
 
-        new_params = self._extract_command_parameters_from_input(
-            input_for_param_extraction,
-            command_parameters_class,
-            stored_missing_fields,
-            self.command_name,
-            app_workflow_folderpath
-        )
+        new_params = input_for_param_extraction.extract_parameters(
+            command_parameters_class, 
+            self.command_name, 
+            app_workflow_folderpath)
 
         if stored_params:
             merged_params = self._merge_parameters(stored_params, new_params, stored_missing_fields)
@@ -488,26 +503,6 @@ class ParameterExtraction:
                 lines.append(f"{display_name}: {value.value}")
         return "\n".join(lines)
 
-    @staticmethod
-    def _extract_command_parameters_from_input(
-        input_for_param_extraction: BaseModel,
-        command_parameters_class: Type[BaseModel],
-        missing_fields: list = None,
-        app_command_name: str = None,
-        app_workflow_folderpath: str = None,
-    ) -> BaseModel:
-        """
-        Extract command parameters from user input.
-        This implementation handles any parameter type.
-        """
-        if missing_fields:
-            default_params = InputForParamExtraction.populate_defaults_dict(
-                command_parameters_class)
-            return ParameterExtraction._apply_missing_fields(
-                input_for_param_extraction.command, default_params, missing_fields)
-
-        return input_for_param_extraction.extract_parameters(command_parameters_class, app_command_name, app_workflow_folderpath)
-
     @staticmethod
     def _apply_missing_fields(command: str, default_params: BaseModel, missing_fields: list):
         params = default_params.model_copy()
@@ -580,6 +575,8 @@ class ResponseGenerator:
         if cnp_output.error_msg:
             workflow_context = workflow.context
             workflow_context["NLU_Pipeline_Stage"] = NLUPipelineStage.INTENT_AMBIGUITY_CLARIFICATION
+            if command not in workflow_context:
+                workflow_context["command"] = command
             workflow.context = workflow_context
             return CommandOutput(
                 command_responses=[
@@ -610,7 +607,9 @@ class ResponseGenerator:
                 command=command,
             )
             command_output = CommandExecutor.perform_action(workflow, startup_action)
-            command_output.command_responses[0].artifacts["command_handled"] = True
+            command_output.command_responses[0].artifacts["command_handled"] = True     
+            # Set the additional attributes
+            command_output.command_name = cnp_output.command_name
             return command_output
         
         if nlu_pipeline_stage in {
