@@ -1,23 +1,24 @@
 import sys
 import dspy
 import os
-from typing import Annotated, Optional, Tuple, Union, Dict, Any, Type, List, get_args
+from contextlib import suppress
+from typing import Optional, Tuple, Union, Dict, Any, Type, List, get_args
 from enum import Enum
-from pydantic import BaseModel, Field, ValidationError, field_validator, ConfigDict
-from pydantic_core import PydanticUndefined
-import fastworkflow
-
 from datetime import date
 import re
-import inspect
 from difflib import get_close_matches
-from fastworkflow import ModuleType
-from fastworkflow.utils.pydantic_model_2_dspy_signature_class import TypedPredictorSignature
-from dspy.teleprompt import LabeledFewShot
 import json
+
+from pydantic import BaseModel, ConfigDict
+from pydantic_core import PydanticUndefined
+
+import fastworkflow
+from fastworkflow import ModuleType
 from fastworkflow.utils.logging import logger
-from fastworkflow.model_pipeline_training import train,get_route_layer_filepath_model
-from fastworkflow.utils.fuzzy_match import find_best_match
+from fastworkflow.model_pipeline_training import get_route_layer_filepath_model
+from fastworkflow.utils.fuzzy_match import find_best_matches
+from fastworkflow.command_directory import CommandDirectory
+from fastworkflow.utils.command_dependency_graph import get_dependency_suggestions
 
 MISSING_INFORMATION_ERRMSG = None
 INVALID_INFORMATION_ERRMSG = None
@@ -78,26 +79,29 @@ class DatabaseValidator:
         if not key_values:
             return False, None, []     
 
-        best_match,_=find_best_match(value, key_values,threshold=0.7)
-        if best_match:
-                return True, best_match, []
-        normalized_value = value.lower()
+        match = next((v for v in key_values if value.lower() == v.lower()), None)
+        if match is not None:
+            return True, match, []
 
-        for value in key_values:
-            if normalized_value == value.lower():
-                return True, value, []
+        best_matches, _ = find_best_matches(value, key_values, threshold = 0.7)
+        if len(best_matches) == 1:
+            return True, best_matches[0], []
+        elif len(best_matches) > 1:
+            return False, None, best_matches
 
-        matches = get_close_matches(normalized_value, 
-                                [val.lower() for val in key_values], 
-                                n=3, 
-                                cutoff=threshold)
+        lowercase_matches = get_close_matches(
+            value.lower(), 
+            [val.lower() for val in key_values], 
+            n = 3, 
+            cutoff = threshold
+        )
 
-        original_matches = []
-        for match_lower in matches:
-            for value in key_values:
-                if value.lower() == match_lower:
-                    original_matches.append(value)
-                    break
+        original_matches = [
+            value 
+            for match in lowercase_matches 
+            for value in key_values 
+            if value.lower() == match
+        ]
 
         if original_matches:
             return False, None, original_matches
@@ -110,6 +114,7 @@ class InputForParamExtraction(BaseModel):
 Today's date is {today}.
 """  
     command: str
+    input_for_param_extraction_class: Optional[Type[Any]] = None
     input_for_param_extraction: Optional[Any] = None
     model_config = ConfigDict(arbitrary_types_allowed=True)
     
@@ -143,6 +148,7 @@ Today's date is {today}.
         
         return cls(
             command=subject_command,
+            input_for_param_extraction_class=input_for_param_extraction_class,
             input_for_param_extraction=input_for_param_extraction
         )
     
@@ -232,34 +238,6 @@ Today's date is {today}.
             instructions = generated_docstring
 
             return dspy.Signature(signature_components, instructions)
-
-    @staticmethod
-    def populate_defaults_dict(command_parameters_class):
-        default_params = {}
-        for field_name, field_info in command_parameters_class.model_fields.items():
-            if (
-                field_info.default is not PydanticUndefined and 
-                field_info.default is not None and 
-                field_info.default is not Ellipsis
-            ):
-                default_params[field_name] = field_info.default
-            # Handle strings
-            elif field_info.annotation == str:
-                default_params[field_name] = NOT_FOUND
-            elif field_info.annotation == int:
-                default_params[field_name] = INVALID_INT_VALUE
-            elif field_info.annotation == float:
-                default_params[field_name] = INVALID_FLOAT_VALUE
-            # Handle Optional[int] and Optional[float]
-            elif (hasattr(field_info.annotation, "__origin__") and
-                  field_info.annotation.__origin__ is Union and
-                  (int in field_info.annotation.__args__ or float in field_info.annotation.__args__) and
-                  type(None) in field_info.annotation.__args__):
-                default_params[field_name] = None
-            else:
-                default_params[field_name] = None
-
-        return command_parameters_class(**default_params)
     
     def extract_parameters(self, CommandParameters: Type[BaseModel] = None, subject_command_name: str = None, workflow_folderpath: str = None) -> BaseModel:
         """
@@ -315,12 +293,11 @@ Today's date is {today}.
             except Exception as exc:
                 logger.warning(PARAMETER_EXTRACTION_ERROR_MSG.format(error=exc))
 
-        try:            
-            params = model_class(**param_dict)
-        except ValidationError:
-            params = self.populate_defaults_dict(model_class)
-
-        return params
+        # IMPORTANT: Do *not* instantiate the original model via the regular
+        # constructor – that would invoke full validation including regex
+        # pattern checks.  Instead use `model_construct`, which builds the
+        # object in-place without any validation.
+        return model_class.model_construct(**param_dict)  # type: ignore[arg-type]
     
     def validate_parameters(self,
                             app_workflow: fastworkflow.Workflow, 
@@ -342,10 +319,6 @@ Today's date is {today}.
         invalid_fields = []
         all_suggestions = {}
 
-        # check if the input for parameter extraction class is defined in the registary then call the process_parameters function on the instance.
-        if hasattr(self.input_for_param_extraction, 'process_extracted_parameters'):
-            self.input_for_param_extraction.process_extracted_parameters(app_workflow, subject_command_name, cmd_parameters)
-
         # Check required fields
         for field_name, field_info in type(cmd_parameters).model_fields.items():
             field_value = getattr(cmd_parameters, field_name, None)
@@ -413,10 +386,13 @@ Today's date is {today}.
                 is_db_lookup = field_info.json_schema_extra.get("db_lookup")
 
             if is_db_lookup:
-                if not self.input_for_param_extraction:
-                    raise ValueError("input_for_param_extraction is not set.")
-                key_values=self.input_for_param_extraction.db_lookup(app_workflow, subject_command_name) 
-                matched, corrected_value, field_suggestions = DatabaseValidator.fuzzy_match(field_value, key_values)
+                if not self.input_for_param_extraction_class:
+                    raise ValueError("input_for_param_extraction_class is not set.")
+                matched, corrected_value, field_suggestions = (
+                    self.input_for_param_extraction_class.db_lookup(
+                        app_workflow, field_name, field_value)
+                ) 
+                # matched, corrected_value, field_suggestions = DatabaseValidator.fuzzy_match(field_value, key_values)
 
                 if matched:
                     setattr(cmd_parameters, field_name, corrected_value)
@@ -426,16 +402,40 @@ Today's date is {today}.
                     is_valid = False
 
         if is_valid:
-            return (True, "All required parameters are valid.", {})
-
-        message = ""
+            if not (
+                self.input_for_param_extraction_class and \
+                hasattr(self.input_for_param_extraction_class, 'validate_extracted_parameters')
+            ):
+                return (True, "All required parameters are valid.", {})
+            
+            is_valid, message = self.input_for_param_extraction_class.validate_extracted_parameters(app_workflow, subject_command_name, cmd_parameters)
+            if is_valid:
+                return (True, "All required parameters are valid.", {})
+            return (False, message, {})
 
+        message = ''
         if missing_fields:
             message += f"{MISSING_INFORMATION_ERRMSG}" + ", ".join(missing_fields) + "\n"
 
         if invalid_fields:
             message += f"{INVALID_INFORMATION_ERRMSG}" + ", ".join(invalid_fields) + "\n"
 
+        with suppress(Exception):
+            graph_path = os.path.join(CommandDirectory.get_commandinfo_folderpath(app_workflow.folderpath), "parameter_dependency_graph.json")
+            suggestions_texts: list[str] = []
+            for field in missing_fields:
+                plans = get_dependency_suggestions(graph_path, subject_command_name, field, min_weight=0.7, max_depth=3)
+                if plans:
+                    # Format a concise plan: main command and any immediate sub-steps
+                    top = plans[0]
+                    def format_plan(p):
+                        if not p.get('sub_plans'):
+                            return p['command']
+                        return p['command'] + " -> " + " -> ".join(sp['command'] for sp in p['sub_plans'])
+                    suggestions_texts.append(f"To get '{field}', try: {format_plan(top)}")
+            if suggestions_texts:
+                message += "\n" + "\n".join(suggestions_texts) + "\n"
+
         for field, suggestions in all_suggestions.items():
             if suggestions:
                 is_format_instruction = any(("format" in str(s).lower() or "pattern" in str(s).lower()) for s in suggestions)
@@ -443,7 +443,7 @@ Today's date is {today}.
                 if is_format_instruction:
                     message += f"\n{field}: {', '.join(suggestions)}"
                 else:
-                    message += f"\nDid you mean one of these {field}s? {', '.join(suggestions)}"
+                    message += f"Did you mean one of these {field}s? {', '.join(suggestions)}\n"
 
         combined_fields = []
         if missing_fields:
