@@ -2,8 +2,7 @@ import os
 import shutil
 import sys
 from functools import wraps
-from typing import Any, Optional
-from pydantic import BaseModel
+from typing import Optional
 
 from speedict import Rdict
 
@@ -37,29 +36,45 @@ def enablecache(func):
 class Workflow:
     """Workflow class"""
     @classmethod
-    def create(cls, workflow_folderpath: str, workflow_id_str: Optional[str] = None, parent_workflow_id: Optional[int] = None, workflow_context: dict = None) -> "Workflow":
+    def create(
+        cls, 
+        workflow_folderpath: str, 
+        workflow_id_str: Optional[str] = None, 
+        parent_workflow_id: Optional[int] = None, 
+        workflow_context: dict = None,
+        project_folderpath: Optional[str] = None
+    ) -> "Workflow":
         if workflow_id_str is not None and parent_workflow_id is not None:
             raise ValueError("workflow_id_str and parent_workflow_id cannot both be provided")
 
-        if workflow_id_str:
-            workflow_id = fastworkflow.get_workflow_id(workflow_id_str)
-        else:
-            workflow_id = cls.generate_child_workflow_id(workflow_folderpath, parent_workflow_id)
-
-        if workflow := cls.get_workflow(workflow_id):
-            if workflow_context is not None:
-                workflow.context = workflow_context
-            return workflow
-
         if not os.path.exists(workflow_folderpath):
             raise ValueError(f"The folder path {workflow_folderpath} does not exist")
 
         if not os.path.isdir(workflow_folderpath):
             raise ValueError(f"{workflow_folderpath} must be a directory")
 
+        if workflow_id_str:
+            workflow_id = fastworkflow.get_workflow_id(workflow_id_str)
+        else:
+            workflow_id = cls.generate_child_workflow_id(workflow_folderpath, parent_workflow_id)
+
+        python_project_path = project_folderpath or workflow_folderpath
+        if python_project_path not in sys.path:
+            # THIS IS IMPORTANT: it allows relative import of modules in the code inside workflow_folderpath
+            sys.path.insert(0, python_project_path)
+
+        if workflow := cls.get_workflow(workflow_id):
+            if workflow_context is not None:
+                workflow.context = workflow_context
+            return workflow
+
+        # Resolve the workflow path to ensure consistent cache keys
+        from pathlib import Path
+        resolved_workflow_path = str(Path(workflow_folderpath).resolve())
+        
         workflow_snapshot = {
             "workflow_id": workflow_id,
-            "workflow_folderpath": workflow_folderpath,
+            "workflow_folderpath": resolved_workflow_path,
             "workflow_context": workflow_context or {},
             "parent_workflow_id": parent_workflow_id,
             "is_complete": False
@@ -136,8 +151,10 @@ class Workflow:
         if create_key is not Workflow.__create_key:
             raise ValueError("Workflow objects must be created using Workflow.create")
 
+        from pathlib import Path
         self._id = workflow_snapshot["workflow_id"]
-        self._folderpath = workflow_snapshot["workflow_folderpath"]
+        # Always resolve the workflow path to ensure consistent cache keys
+        self._folderpath = str(Path(workflow_snapshot["workflow_folderpath"]).resolve())
         self._parent_id = workflow_snapshot.get("parent_workflow_id")
         self._is_complete = workflow_snapshot.get("is_complete", False)
         self._context = workflow_snapshot.get("workflow_context", {}) 
@@ -156,10 +173,6 @@ class Workflow:
         self._save()
         self._dirty = False
 
-        if self._folderpath not in sys.path:
-            # THIS IS IMPORTANT: it allows relative import of modules in the code inside workflow_folderpath
-            sys.path.insert(0, self._folderpath)
-
     @property
     def current_command_context(self) -> object:
         return self._current_command_context
@@ -170,8 +183,17 @@ class Workflow:
 
     @property
     def current_command_context_displayname(self) -> str:
+        crd = fastworkflow.CommandContextModel.load(self._folderpath)
+        context_class = crd.get_context_class(
+                Workflow.get_command_context_name(self._current_command_context),
+                fastworkflow.ModuleType.CONTEXT_CLASS
+        )
+        if context_class and hasattr(context_class, 'get_displayname'):
+            return context_class.get_displayname(self._current_command_context)
+
         return Workflow.get_command_context_name(self._current_command_context, for_display=True)
 
+
     @property
     def is_current_command_context_root(self) -> bool:
         return self._current_command_context == self._root_command_context
